/*
    bookmark.prg

    A simple memoeditor that allows users to set bookmarks in the memo
    text, and return to the positions in the memo where the bookmarks are
    set.  Provides an array tbrowse of the line of text as it appeared
    when the user pressed the key to set a bookmark.

    This sample uses K_F7 to set a bookmark, and K_F8 to display the
    browser of existing bookmarks for the user to choose the one that
    represents where he wants to go in the memo.

    Copyright 1994-1995 by Extrasensory Software.  Permission granted
    to use and distribute these functions freely in source, object, or
    executable form as long as this copyright notice is not removed 
    from the source.
*/
#include "inkey.ch"
#include "mredit.ch"

static ele := 1  // used in bookmark array browser


procedure bookmark(memo)
	local nKey
	local lShift
	local lCtrl
	local clip := ""
	local aBookmark := {}
	local nBookmark 
	local clearBlock := .t.


	do while (nKey != K_ESC .and. nKey != K_F10)

		setcursor(iif(meSetInsert(memo),2,1))

		nKey   := inkey(0)
		if clearBlock
			meBlockOff(memo)
			clearBlock := .f.
		endif

		setcursor(0)

		// Clipper won't give us the state of the Shift and Ctrl keys
		// so we have to improvise and grab them here. Shift is used to
		// implement Shift+Arrow marking. Ctrl is used to make sure that
		// keys like Ctrl+C and PageDown can be told apart.
		lShift := meIsShift()
		lCtrl  := meIsCtrl()

		do case	
			case nKey >= 32 .and. nKey <= 255 .or. nKey == 13
				meapplykey( memo, nKey )

			case nKey == K_INS .and. !lCtrl
				meSetInsert(memo, !meSetInsert(memo))

			case nKey == K_UP .and. !lCtrl
				// If the shift key is down and there is not already a block
				// marked start marking a block
				if lShift .and. !meIsBlock( memo )
					meBlockOn( memo )
				endif
				meUp( memo )

				// we don't want this to fire on Ctrl+X, so don't   	
				// do this if the Ctrl key was down!
			case nKey == K_DOWN .and. !lCtrl
				if lShift .and. !meIsBlock( memo )
					meBlockOn( memo, BLOCK_EXCLUSIVE )
				endif
				meDown( memo )

			case nKey == K_LEFT .and. !lCtrl
				if lShift .and. !meIsBlock( memo )
					meBlockOn( memo, BLOCK_EXCLUSIVE )
				endif
				meLeft( memo )

			case nKey == K_RIGHT .and. !lCtrl
				if lShift .and. !meIsBlock( memo )
					meBlockOn( memo, BLOCK_EXCLUSIVE )
				endif
				meRight( memo )

			case nKey == K_PGUP .and. !lCtrl
				mePageUp( memo )

				// we don't want this to fire on Ctrl+C, so don't   	
				// do this if the Ctrl key was down!
			case nKey == K_PGDN .and. !lCtrl
				mePageDown( memo )

			case nKey == K_HOME .and. !lCtrl
				meHome( memo )

			case nKey == K_END .and. !lCtrl
				meEnd( memo )

			case nKey == K_CTRL_RIGHT .and. lCtrl
				meWordRight( memo )

			case nKey == K_CTRL_LEFT .and. lCtrl
				meWordLeft( memo )

			case nKey == K_CTRL_HOME .and. lCtrl
				meGoTop( memo )

			case nKey == K_CTRL_END .and. lCtrl
				meGoBottom( memo )

			case nKey == K_BS
				meBackSpace( memo )

			case nKey == K_DEL .and. !lCtrl
				meDelete( memo )

			case nKey == K_CTRL_U .and. lCtrl
				if meIsBlock( memo )
					meBlockOff( memo )
				endif

				// in the following three cases check to see the Ctrl key
				// was down so we don't get false hits.
			case nKey == K_CTRL_X .and. lCtrl
				// Cut: Capture the text to "clip"
				//      Then delete the marked text
				if meIsBlock( memo )
					clip := megetMarkedBlock( memo )
					medelete( memo )
				endif

			case nKey == K_CTRL_C .and. lCtrl
				// Copy: Capture the text to "clip"
				//       Then turn off the block
				if meIsBlock( memo )
					clip := megetMarkedBlock( memo )
					meBlockOff( memo )
				endif

			case nKey == K_CTRL_V .and. lCtrl
				// Paste: Insert the current scrap
				if !empty( clip )
					meInsertText( memo, clip )
				endif

			case nKey == K_F7
				nBookmark := meSetBookmark(memo)
				if nBookmark == -1
					alert("All bookmarks are in use!")
				else
					aadd(aBookmark, {nBookmark, meGetLine(memo, .t. ), ;
							meSetTop(memo)+meWinRow(memo)})
				endif

			case nKey == K_F8
				if empty(aBookmark)
					alert("No bookmarks have been set!")
				else
					BrowseBookmarks(memo, aBookmark)
				endif
				meFixCursor(memo)

		endcase
	enddo
	return 


/*  Begin Bookmark Browse Stuff  */

procedure BrowseBookmarks(memo, aBookmark)
  local cBuff    := savescreen(2,2,8,77)
  local oCol
  local oldColor := setcolor("B+/W,GR+/B,,,W+/BG")
  local oBrow    := tbrowsenew(2,2,8,77)
  local max_ele  := len(aBookmark)
  local key      := 0
  local cHeader  := padc("Highlight a Bookmark, "+;
                         "Press <Enter> to Go There",74)


  ele := 1 // filewide static
  oCol := tbcolumnnew(cHeader,;
                      {||aBookmark[ele,2]})
  oCol:width := 74

  oBrow:addColumn(oCol)

  oBrow:skipBlock     := { |SkipCnt| ArrSkip(aBookmark,SkipCnt, max_ele) }
  oBrow:goTopBlock    := { | | ele := 1 }
  oBrow:goBottomBlock := { | | ele := max_ele }
  oBrow:headsep       := ""


  while key != K_ESC

    dispbegin()
    do while ((key := inkey()) == 0) .and. !oBrow:stabilize()
    enddo
    dispend()

    if oBrow:stable 

      key := inkey(0)

    endif

    if !standardkeys(oBrow, key)
      do case
        case key == K_ENTER
          meDispBegin(memo)
          meGotoBookmark(memo, aBookmark[ele,1])
          meScroll(memo, meSetTop(memo)+meWinRow(memo) - aBookmark[ele,3])
          meDispEnd(memo)
          exit
        case key == K_DEL
          meFreeBookmark(memo, aBookmark[ele,1])
          adel(aBookmark,ele)
          if --max_ele == 0
            asize(aBookmark,0)
            exit
          else
            asize(aBookmark, max_ele)
            oBrow:refreshAll()
          endif
      endcase

    endif
  enddo

  setcolor(oldColor)
  restscreen(2,2,8,77,cBuff)
  return


static function ArrSkip(aArray_,nHowMany, nMax)
  local nActual := nHowMany

  if (ele + nHowMany) < 1
    nActual := ( -ele + 1)
  elseif (ele +nHowMany) > len(aArray_)
    nActual := ( nMax - ele )
  endif
  ele += nActual
return nActual

static function standardKeys(oBrow, key)
  local lReturn := (key == K_ESC) 

  if !lReturn
  // note: no left/right options.  I don't want them to pan this browser
    if (lReturn := (key == K_UP))
      oBrow:up()
    elseif (lReturn := (key == K_DOWN))
      oBrow:down()
    elseif (lReturn := (key == K_PGUP))
      oBrow:pageUp()
    elseif (lReturn := (key == K_PGDN))
      oBrow:pageDown()
    elseif (lReturn := (key == K_CTRL_PGUP))
      oBrow:goTop()
    elseif (lReturn := (key == K_CTRL_PGDN))
      oBrow:goBottom()
    endif
  endif
  return lReturn
 	/* end standardKeys() */

/*  End Bookmark Browse Stuff  */

/* end bookmark.prg */
